<?php
/**
 * ListBox/CheckListbox crée dynamiquement
 * 
 * + Avec ou sans entête
 * + Avec sous sans pied de page
 * + Pied de page comportant un lien pour tout activer et tout désactiver
 * + Entête, pied de page affiché à l'endroit souhaité
 * + Avec ou sans les cases à cocher
 * + Personnalisation de l'apparence via css et images
 * + Taille paramétrable
 * + Recuperation simple du post et récupération sur une autre page
 * + Plusieurs listbox possible
 * + Compatible IE, MOZILLA, FIREFOX, OPERA
 * + Formatage personnalisé des dates et valeurs numériques
 * + Pas besoin de référence le CSS et le JS du script. 
 * 
 * @author stailer - 20 / 03 / 06
 * @version 1.0.0.5
 * @package Fonctions
 * 
 */
class STListBox
{
	/**
	 * Nom de la liste
	 *
	 * @var string
	 */
	var $name = '';
	
	/**
	 * Tableau de données à 2 dimensions
	 * 1=nom de colonne   =>   2=valeur  
	 *
	 * @var array
	 */
	var $datas = null;

	/**
	 * Tableau de données à 1 dimension contenant les id des items cochés
	 *
	 * @var array
	 */
	var $STCheckedList = null;
	
	/**
	 * Tableau d'entête à 2 dimensions.
	 * 1=titre   =>   2=taille en pixel
	 * 
	 * Si non valorisé, il n'y aura pas d'entête avec tri
	 *
	 * @var array
	 */
	var $titles = null;
	
	/**
	 * Longueur totale du tableau.
	 * Il n'y aura pas de scrollbar horizontale
	 *
	 * @var int
	 */
	var $width = 0;
	
	/**
	 * Hauteur totale du tableau.
	 * Il y aura une scrollbar verticale
	 *
	 * @var string
	 */
	var $height = 0;
	
	/**
	 * Précise si les checkbox doivent s'afficher pour créer une sélection
	 *
	 * @var bool
	 */
	var $checkbox = false;
	
	/**
	 * Taille en px de la fonte utilisée
	 *
	 * @var int
	 */
	var $fontsize= 16;
	
	/**
	 * Retour chariot
	 *
	 * @var string
	 */
	var $chr = "\n";
	
	/**
	 * Tableau de format pour les colonnes numérique. Remplira les number_format
	 * exemple : $this->number_format =array(  
	 * 		n°de colonne=>array(separateur de 10aine, 
	 * 							separateur de millier, 
	 * 							symbole à la fin,
	 * 							style html) );
	 * @var array
	 */
	var $number_format= null;
	
	/**
	 * exemple : $this->date_format= array( N°de colonne=>array(
	 * 					array('en-fr','text-align:left')  ) ; 
	 *
	 * @var array
	 */
	var $date_format= null;
	
	/**
	 * style par défaut des colonnes
	 *
	 * @var string
	 */
	var $style_colonne ='';
	
	/**
	 * Sert à déclarer le répertoire du fichier CSS et JS
	 *
	 * @var string
	 */
	var $dir_js_css ='';
	
	/**
 	* Vérifie les variables postées à la recherche d'une listbox, selon son nom passé en paramètre $name
 	*
 	* @param string $name
 	* @return array
 	*/
	function STListBoxPost($name='')
	{
		$result = false;
		foreach ($_POST as $key=>$value)
		{
			$tab = explode($name.'_',$key);
			if ($tab && count($tab) == 2)
			{
				$result[] = $tab[1];
			}
		}

		return $result;
	}

	/**
	 * Nouvelle instanciation du listbox
	 *
	 * @param string $name
	 * @param array $datas
	 * @param $dir_js_css répertoire des fichiers CSS et JS de STListBox
	 * @return STListBox
	 */
	function STListBox($name=null, $datas=null, $dir_js_css='', $checkedlist=null)
	{	
		$this->dir_javascript = $dir_js_css;
		$this->name = $name;
		$this->datas = $datas;
		$this->STCheckedList = $checkedlist;
		$this->titles = null;
	}
	
	
	
	/**
	 * Ouvre simplement un script javascript et défini le head de la page
	 * Utilisez JS_RegisterClose pour fermer ce script
	 *
	 * @return string
	 */
	function JS_RegisterOpen()
	{
		$script  = '<script type="text/javascript">'.$this->chr;
		$script .= 'HEAD = document.getElementsByTagName("head");'.$this->chr;
		return $script;
	}
	
	/**
	 * Ferme simplement un script javascript
	 * Utilisez JS_RegisterOpen pour l'ouvrir
	 *
	 * @return string
	 */
	function JS_RegisterClose()
	{
		return '</script>';
	}
	
	/**
	 * Référence le fichier CSS ou JS passé en paramètre dans le heade de la page.
	 * Le répertoire de ce fichier est donnée à l'instanciation de la classe
	 * 
	 * $type peut prendre 'css' ou 'javascsript'
	 *
	 * @param string $type
	 * @param string $fichier
	 * @return string
	 */
	function JS_RegisterScript($type='javascript', $fichier='')
	{
		if (!is_file($this->dir_javascript.'/'.$fichier))
		{
			echo 'STListBox => Le fichier <b>'.$this->dir_javascript.'/'.$fichier.'</b> n\'existe pas ou n\'est pas dans ce répertoire';
			exit;
		}
		else 
		{
			switch ($type)
			{
				case  'javascript' :
					{
						$script  = 'SCRIPT = document.createElement("script");'.$this->chr;
						$script .= 'SCRIPT.src  = "'.$this->dir_javascript.'/'.$fichier.'";'.$this->chr;
					}
					break;
				case 'css' :
					{
						$script  = 'SCRIPT = document.createElement("link");'.$this->chr;
						$script .= 'SCRIPT.rel = "stylesheet";'.$this->chr;
						$script .= 'SCRIPT.href  = "'.$this->dir_javascript.'/'.$fichier.'";'.$this->chr;
					}
					break;
			}
			$script .= 'SCRIPT.type = "text/'.strtolower($type).'";'.$this->chr;
			$script .= 'HEAD[0].appendChild(SCRIPT);'.$this->chr;
			return $script;
		}
	}
	
	
	/**
	 * Ajoute dynamiquement la référence au CSS et au JS dans le HEAD de la page (X)HTML
	 *
	 * @return string
	 */
	function STListBoxRegister()
	{
		global $STListBoxRegister;
		if (!isset($STListBoxRegister))
		{
			$STListBoxRegister  = $this->JS_RegisterOpen();	 
			$STListBoxRegister .= $this->JS_RegisterScript('javascript','sorttable.js');
			$STListBoxRegister .= $this->JS_RegisterScript('css','sorttable.css');
			$STListBoxRegister .= $this->JS_RegisterClose();
			return $STListBoxRegister;
		}
		else 
			return '';
		
	}
	
	/**
	 * Création du header avec entête cliquable
	 *
	 * @return string
	 */
	function STListBoxHeader()
	{
		// CREATION TABLE D'ENTETE ---- + si $titles valorisé, et liée avec la ligne vide de tri sur le tableau de données
		if (count($this->titles)>0)
		{
			$header = '<table style="width:'.intval($this->width-16).'px" class="sortable_header"  cellspacing="0" >'
						.$this->chr.'<tr>';
			
			$cpt = 0;
			foreach ($this->titles as $title=>$taille)
			{
				$header.=
				'<th id="header_'.
				$this->name.'_'.$cpt.'" style="width:'.$taille.'px;"onclick="ts_resortTable(\''.
				$this->name.'\' ,'.$cpt.' , document.getElementById(\'link_'.$this->name.'_'.$cpt.'\')) ">'.
				$title.'</th>';

				$cpt++;
			}
			$header.='</tr>';
			$header.='</table>'.$this->chr;
			
			return $this->STListBoxRegister().$header;
		}
		else 
			return false;
	}

	/**
	 * Méthode de rendu final
	 *
	 * @return string
	 */
	function STListBoxContent()
	{
		// VERIFICATION DES DONNEES
		if (count($this->datas)<=0) return false;
		
		// DEFINITION DU STYLE DU DIV CONTENEUR
		$result ='<div style="padding:0px;overflow:scroll;overflow-y:scroll;overflow-x:hidden;overflow:-moz-scrollbars-vertical;width:'.
		$this->width.'px;height:'.
		$this->height.'px;
		border-right:#EFEFEF 1px solid;border-bottom:#EFEFEF 1px solid;border-top:#EFEFEF 1px solid;border-left:#EFEFEF 1px solid;font-size: '.
		$this->fontsize.'px;">'.$this->chr;



		// CREATION DE LA TABLE HTML ---- + avec ligne de tri si $titles valorisé
		if (count($this->titles)>0)
		{
			$result .= '<table id="'.$this->name.'" class="sortable" cellspacing="0" >'.$this->chr;
			$result.='<tr>';
			foreach ($this->titles as $title=>$taille)
			{
				$result.='<th style="width:'.$taille.'px;">'.$title.'</th>';
			}
			$result.='</tr>';
		}
		else
		$result .= '<table id="'.$this->name.'" cellspacing="0" >'.$this->chr;



		// CREATION DES LIGNES
		$cpt = 0;
		while ($cpt < count($this->datas))
		{
			$index = '';
			$start = true;
			if (!$this->checkbox) $result.= '  <tr>'.$this->chr;

			$col = 0;
			foreach ($this->datas[$cpt] as $key=>$values)
			{
				if ($this->checkbox && $start) // première colonne
				{
					$index = $this->name.'_'.$values;
					
					if ($this->getChecked($cpt))
					{
						$class = 'class="sortable_row_on"';
						$check = 'checked="checked"';
					}
					else
					{
						$class = 'class="sortable_row_off"';
						$check = '';
					}
					$result.= '<tr onclick="checkChange('.$values.',\''.$index.'\');" '.$class.' id="'.$index.'">'.$this->chr.
					'<td><input value="'.$values.'" '.
					$check.' onclick="checkChange('.$values.',\''.$index.'\');" name="'.$index.'" type="checkbox" class="listbox_box" />';
					
					$start = false;
				}
				else 
				{
					// formatage des colonnes
					
					// cas du numérique
					if (isset($this->number_format[$col]))
					{
						$values = number_format($values,2,			
								$this->number_format[$col][0],//séparateur de dizaine
							 	$this->number_format[$col][1]). // séparateur de milliers
								 $this->number_format[$col][2]; // symbole
						
						$result.= '<td style="'.$this->number_format[$col][3].'">'.$values;
					}
					
					// cas de date
					else if (isset($this->date_format[$col]))
					{
						switch ($this->date_format[$col][0])
						{
							case 'fr-en':
							{
								list ($jour , $mois , $an) = split('/',$values);
								$values = $an.'-'.$mois.'-'.$jour;
							}
							break;
							case 'en-fr':
							{
								list ($an , $mois , $jour) = split('-',$values);
								$values = $jour.'/'.$mois.'/'.$an;
							}
							break;
						}
						$result.= '<td style="'.$this->date_format[$col][1].'">'.$values;
					}
					
					// sinon traite comme une chaine
					else
					{
						$result.= '<td style="'.$this->style_colonne.'">'.$values;
					}
				}

				$col++;
				$result.= '</td>';
			}

			$cpt ++;
			$index = '';
			$result.= '</tr>'.$this->chr;
		}


		// FIN ET RETOUR
		$result .= '</table>'.$this->chr;
		$result.= '</div>'.$this->chr;

		return $this->STListBoxRegister().$result;
	}
	
	/**
	 * Création du bas du ListBox
	 * $txtAll et $txtNone sont les phrases à afficher pour les liens permettant de tout cocher ou tout décocher.
	 * Utilisé dans le cadre d'une application multilangue
	 * 
	 * @param string $txtAll
	 * @param string $txtNone
	 * 
	 * @return string
	 */
	function STListBoxFooter($txtItem)
	{
		// CREATION TABLE FOOTER 
		$footer = '<table class="sortable_footer" style="width:'.intval($this->width-16).'px;" cellspacing="0" >'.
		$this->chr.'<tr><td>';
		
		if ($this->checkbox)
		{
            $i = 0;
            while (isset($txtItem[$i]['item'])) {
			    $footer.='<a class="linkfooter" onclick="checkChangeLink(\''.$this->name.'\',\''.$txtItem[$i]['item'].'\');">'.$txtItem[$i]['title'].'</a>&nbsp;';
                $i++;
            }

		}
			
		$footer.='</td></tr>';
		$footer.='</table>'.$this->chr;
			
		return $this->STListBoxRegister().$footer;
	}
	
	/**
	 * Ajoute un élément au tableau indiquant si la ligne est cochée ou non par défaut
	 *
	 * @param bool $bool
	 */
	function setChecked($bool)
	{
		$this->STCheckedList[] = $bool;	
	}
	
	/**
	 * Vérifie un élément du tableau et retourne s'il est coché ou non
	 *
	 * @param int $ref
	 * @return bool
	 */
	function getChecked($ref)
	{
		return $this->STCheckedList[$ref]; 
	}
	
}
?>